/*
 * gsampler-client-channel.c
 * Copyright (C) Lilian Monchalin 2011 <isaya07@users.sf.net>
 * 
 * gsampler is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * gsampler is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <string.h>

#include "gsampler-client-channel.h"
#include "gsampler-utils.h"

/**
 * Channel info...
 * 
 **/
void 
gsampler_channel_info_copy (const GsamplerChannelInfo* self, GsamplerChannelInfo* dest) 
{
	dest->id = self->id;
	dest->audio = self->audio;
	dest->audio_channels = self->audio_channels;
	dest->audio_routing = g_strdup (self->audio_routing);
	dest->midi = self->midi;
	dest->file = g_strdup (self->file);
	dest->instrument = self->instrument;
	dest->inst_name = g_strdup (self->inst_name);
	dest->inst_status = self->inst_status;
	dest->engine = g_strdup (self->engine);
	dest->volume = self->volume;
	dest->midi_port = self->midi_port;
	dest->midi_chan = g_strdup (self->midi_chan);
	dest->midi_map = g_strdup (self->midi_map);
	dest->mute = self->mute;
	dest->solo = self->solo;
}

void 
gsampler_channel_info_destroy (GsamplerChannelInfo* self) 
{
	g_free (self->file);
	g_free (self->audio_routing);
	g_free (self->inst_name);
	g_free (self->engine);
	g_free (self->midi_chan);
	g_free (self->midi_map);
}

GsamplerChannelInfo* 
gsampler_channel_info_dup (const GsamplerChannelInfo* self)
{
	GsamplerChannelInfo* dup;
	dup = g_new0 (GsamplerChannelInfo, 1);
	gsampler_channel_info_copy (self, dup);
	return dup;
}

void 
gsampler_channel_info_free (GsamplerChannelInfo* self)
{
	gsampler_channel_info_destroy (self);
	g_free (self);
}

/**
 * Channel Routine...
 * 
 **/

gint 
gsampler_client_create_channel (GsamplerClient* self) 
{
	g_return_val_if_fail (self != NULL, 0);

	return gsampler_client_create (self, "ADD CHANNEL");
}

gboolean 
gsampler_client_delete_channel (GsamplerClient* self, gint id) 
{
	g_return_val_if_fail (self != NULL, FALSE);

	return gsampler_client_send_bool (self, g_strdup_printf ("REMOVE CHANNEL %i", id));
}

gchar** 
gsampler_client_get_available_channel (GsamplerClient* self) 
{
	g_return_val_if_fail (self != NULL, NULL);
	
	return gsampler_client_get_list (self, "LIST CHANNELS");
}

gchar** 
gsampler_client_get_available_engine_list (GsamplerClient* self) 
{
	g_return_val_if_fail (self != NULL, NULL);

	return gsampler_client_get_list (self, "LIST AVAILABLE_ENGINES");
}

GsamplerChannelInfo* 
gsampler_client_get_channel_info (GsamplerClient* self, gint id) 
{
	GsamplerChannelInfo* retour;
	gchar* com = NULL;
	GSList* ret;
	g_return_val_if_fail (self != NULL, NULL);

	retour = g_new0 (GsamplerChannelInfo, 1);
	com = g_strconcat ("GET CHANNEL INFO ", g_strdup_printf ("%i", id), NULL);
	ret = gsampler_client_client_get_info (self, com);
	if (g_slist_length (ret) != 0) 
	{
		GSList* ele_it;
		retour->id = id;
		for (ele_it = ret; ele_it != NULL; ele_it = ele_it->next) 
		{
			gchar* ele = NULL;
			gchar* temp = NULL;
			ele = g_strdup ((const gchar*) ele_it->data);
			if (string_contains (ele, "ENGINE_NAME"))
			{
				temp = g_strstrip (string_replace (ele, "ENGINE_NAME:", ""));
				retour->engine = g_utf8_strup (string_replace (temp, "Engine", ""), (gssize) (-1));
			} else if (string_contains (ele, "AUDIO_OUTPUT_DEVICE"))
			{
				temp = g_strstrip (string_replace (ele, "AUDIO_OUTPUT_DEVICE:", ""));
				retour->audio = atoi (temp);
			} else if (string_contains (ele, "AUDIO_OUTPUT_CHANNELS"))
			{
				temp = g_strstrip (string_replace (ele, "AUDIO_OUTPUT_CHANNELS:", ""));
				retour->audio_channels = atoi (temp);
			} else if (string_contains (ele, "AUDIO_OUTPUT_ROUTING"))
			{
				retour->audio_routing = g_strstrip (string_replace (ele, "AUDIO_OUTPUT_ROUTING:", ""));
			} else if (string_contains (ele, "INSTRUMENT_FILE"))
			{
				temp = g_strstrip (string_replace (ele, "INSTRUMENT_FILE:", ""));
				retour->file = string_replace (temp, "\\x20", " ");
			} else if (string_contains (ele, "INSTRUMENT_NR"))
			{
				temp = g_strstrip (string_replace (ele, "INSTRUMENT_NR:", ""));
				retour->instrument = atoi (temp);
			} else if (string_contains (ele, "INSTRUMENT_NAME"))
			{
				temp = g_strstrip (string_replace (ele, "INSTRUMENT_NAME:", ""));
				retour->inst_name = g_strdup (temp);
			} else if (string_contains (ele, "INSTRUMENT_STATUS"))
			{
				temp = g_strstrip (string_replace (ele, "INSTRUMENT_STATUS:", ""));
				retour->inst_status = atoi (temp);
			} else if (string_contains (ele, "MIDI_INPUT_DEVICE"))
			{
				temp = g_strstrip (string_replace (ele, "MIDI_INPUT_DEVICE:", ""));
				retour->midi = atoi (temp);
			} else if (string_contains (ele, "MIDI_INPUT_PORT"))
			{
				temp = g_strstrip (string_replace (ele, "MIDI_INPUT_PORT:", ""));
				retour->midi_port = atoi (temp);
			} else if (string_contains (ele, "MIDI_INPUT_CHANNEL"))
			{
				temp = g_strstrip (string_replace (ele, "MIDI_INPUT_CHANNEL:", ""));
				retour->midi_chan = g_strdup (temp);
			} else if (string_contains (ele, "VOLUME"))
			{
				temp = g_strstrip (string_replace (ele, "VOLUME:", ""));
				retour->volume = double_parse (temp);
			} else if (string_contains (ele, "MUTE"))
			{
				temp = g_strstrip (string_replace (ele, "MUTE:", ""));
				retour->mute = bool_parse (temp);
			} else if (string_contains (ele, "SOLO")) 
			{
				temp = g_strstrip (string_replace (ele, "SOLO:", ""));
				retour->solo = bool_parse (temp);
			} else if (string_contains (ele, "MIDI_INSTRUMENT_MAP")) 
			{
				temp = g_strstrip (string_replace (ele, "MIDI_INSTRUMENT_MAP:", ""));
				retour->midi_map = g_strdup (temp);
			}
			g_free (temp);
			g_free (ele);
		}

	} else 
	{
		retour->id = -1;
	}
	g_free (com);
	g_slist_free_full (ret, g_free);
	return retour;
}

gboolean 
gsampler_client_set_channel_volume (GsamplerClient* self, gint id, gdouble vol)
{
	return gsampler_client_send_bool (self, g_strdup_printf ("SET CHANNEL VOLUME %i %s",
	                                                         id, double_to_string ((vol/100))));
}

gboolean 
gsampler_client_set_channel_mute (GsamplerClient* self, gint id, gint state)
{
	return gsampler_client_send_bool (self, g_strdup_printf ("SET CHANNEL MUTE %i %i", id, state));
}

gboolean 
gsampler_client_set_channel_solo (GsamplerClient* self, gint id, gint state)
{
	return gsampler_client_send_bool (self, g_strdup_printf ("SET CHANNEL SOLO %i %i", id, state));
}

gboolean
gsampler_client_set_channel_midimap (GsamplerClient* self, gint id, gint map)
{
	return gsampler_client_send_bool (self, g_strdup_printf ("SET CHANNEL MIDI_INSTRUMENT_MAP %i %i",
	                                                         id, map));
}

gboolean 
gsampler_client_set_channel_output (GsamplerClient* self, gint id, gint audio)
{
	return gsampler_client_send_bool (self, g_strdup_printf ("SET CHANNEL AUDIO_OUTPUT_DEVICE %i %i",
	                                                         id, audio));
}

gboolean 
gsampler_client_set_channel_input (GsamplerClient* self, gint id, gint midi) 
{
	return gsampler_client_send_bool (self, g_strdup_printf ("SET CHANNEL MIDI_INPUT_DEVICE %i %i",
	                                                         id, midi));
}

gboolean				
gsampler_client_set_channel_audio_output_channel (GsamplerClient* self, gint chan, 
                                                  gint aud_out, gint aud_in)
{
	return gsampler_client_send_bool (self, g_strdup_printf ("SET CHANNEL AUDIO_OUTPUT_CHANNEL %d %d %d", 
	                                                         chan, aud_out, aud_in));
}

gboolean 
gsampler_client_load_instrument_modal (GsamplerClient* self, gint id,
                                       const gchar* file, gint inst) 
{
	return gsampler_client_send_bool (self, g_strdup_printf ("LOAD INSTRUMENT '%s' %i %i",
	                                                         file, inst, id));
}

gboolean 
gsampler_client_load_instrument (GsamplerClient* self, gint id, const gchar* file, gint inst) 
{
	gboolean result = FALSE;
	gchar* com;
	gchar* ret;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (file != NULL, FALSE);

	com = g_strdup_printf ("LOAD INSTRUMENT NON_MODAL '%s' %i %i", file, inst, id);
	ret = gsampler_client_send_commande (self, com);
	if (!string_contains (ret, "ERR")) result = TRUE;
	g_free (ret);
	g_free (com);
	return result;
}

gchar** 
gsampler_client_get_file_instrument_list (GsamplerClient* self, const gchar* file) 
{
	g_return_val_if_fail (file != NULL, FALSE);
	
	return gsampler_client_get_list (self, g_strdup_printf ("LIST FILE INSTRUMENTS '%s'", file));
}

InstrumentInfo *
gsampler_client_get_file_instrument_info (GsamplerClient* self, const gchar* file, gint id) 
{
	InstrumentInfo *result = NULL;
	gchar* com = NULL;
	GSList* ret;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (file != NULL, NULL);

	result = g_new0 (InstrumentInfo, 1);
	com = g_strdup_printf ("GET FILE INSTRUMENT INFO '%s' %i", file, id);
	ret = gsampler_client_client_get_info (self, com);
	if (g_slist_length (ret) != 0) 
	{
		GSList* ele_it;
		for (ele_it = ret; ele_it != NULL; ele_it = ele_it->next) 
		{
			gchar* ele = NULL;
			ele = g_strdup ((const gchar*) ele_it->data);
			if (string_contains (ele, "NAME")) {
				result->name = g_strstrip (string_replace (ele, "NAME:", ""));
			} else if (string_contains (ele, "FORMAT_FAMILY")) {
				result->format = g_strstrip (string_replace (ele, "FORMAT_FAMILY:", ""));
			} else if (string_contains (ele, "FORMAT_VERSION")) {
				result->formatv = g_strstrip (string_replace (ele, "FORMAT_VERSION:", ""));
			} else if (string_contains (ele, "PRODUCT")) {
				result->product = g_strstrip (string_replace (ele, "PRODUCT:", ""));
			} else if (string_contains (ele, "ARTISTS")) {
				result->artists = g_strstrip (string_replace (ele, "ARTISTS:", ""));
			}
			g_free (ele);
		}
	}
	g_slist_free_full (ret, g_free);
	g_free (com);
	return result;
}

gboolean 
gsampler_client_load_engine (GsamplerClient* self, const gchar* engine, gint id)
{
	return gsampler_client_send_bool (self, g_strdup_printf ("LOAD ENGINE %s %i", engine, id));
}

gboolean 
gsampler_client_edit_channel_instrument (GsamplerClient* self, gint id) 
{
	return gsampler_client_send_bool (self, g_strdup_printf ("EDIT CHANNEL INSTRUMENT %i", id));
}

gboolean 
gsampler_client_set_channel_midi_channel (GsamplerClient* self, gint id, const gchar* chan) 
{
	return gsampler_client_send_bool (self, g_strdup_printf ("SET CHANNEL MIDI_INPUT_CHANNEL %i %s",
	                                                         id, chan));
}

gboolean 
gsampler_client_set_channel_midi_port (GsamplerClient* self, gint id, gint port) 
{
	return gsampler_client_send_bool (self, g_strdup_printf ("SET CHANNEL MIDI_INPUT_PORT %i %i",
	                                                         id, port));
}

gboolean 
gsampler_client_send_channel_midi_note_on (GsamplerClient* self, gint id, gint pitch, gint vel) 
{
	if (vel < 0) 
	{
		vel = 0;
	} else if (vel > 127) 
	{
		vel = 127;
	}
	return gsampler_client_send_bool (self, g_strdup_printf ("SEND CHANNEL MIDI_DATA NOTE_ON %i %i %i", 
	                                                         id, pitch, vel));
}

gboolean 
gsampler_client_send_channel_midi_note_off (GsamplerClient* self, gint id, gint pitch, gint vel)
{
	if (vel < 0) 
	{
		vel = 0;
	} else if (vel > 127) 
	{
		vel = 127;
	}
	return gsampler_client_send_bool (self, g_strdup_printf ("SEND CHANNEL MIDI_DATA NOTE_OFF %i %i %i",
	                                                         id, pitch, vel));
}
